@@ -454,6 +454,21 @@ on_sigterm (State *state)
         return G_SOURCE_CONTINUE;
 }
 
+static gboolean
+on_registration_delay_complete (State *state)
+{
+        gboolean ret;
+
+        ret = register_display (state, state->cancellable);
+
+        if (!ret) {
+                g_printerr ("Unable to register display with display manager\n");
+                g_main_loop_quit (state->main_loop);
+        }
+
+        return G_SOURCE_REMOVE;
+}
+
 int
 main (int    argc,
       char **argv)
@@ -528,13 +543,7 @@ main (int    argc,
                 goto out;
         }
 
-        ret = register_display (state, state->cancellable);
-
-        if (!ret) {
-                g_printerr ("Unable to register display with display manager\n");
-                exit_status = EX_SOFTWARE;
-                goto out;
-        }
+        g_timeout_add_seconds (2, (GSourceFunc) on_registration_delay_complete, state);
 
         g_main_loop_run (state->main_loop);
 
@@ -545,9 +554,11 @@ main (int    argc,
         }
 
 out:
-        signal_subprocesses (state);
-        wait_on_subprocesses (state);
-        clear_state (&state);
+        if (state != NULL) {
+                signal_subprocesses (state);
+                wait_on_subprocesses (state);
+                clear_state (&state);
+        }
 
         return exit_status;
 }
