@@ -810,6 +810,21 @@ on_sigterm (State *state)
         return G_SOURCE_CONTINUE;
 }
 
+static gboolean
+on_registration_delay_complete (State *state)
+{
+        gboolean ret;
+
+        ret = register_display (state, state->cancellable);
+
+        if (!ret) {
+                g_printerr ("Unable to register display with display manager\n");
+                g_main_loop_quit (state->main_loop);
+        }
+
+        return G_SOURCE_REMOVE;
+}
+
 int
 main (int    argc,
       char **argv)
@@ -896,14 +911,6 @@ main (int    argc,
                 goto out;
         }
 
-        ret = register_display (state, state->cancellable);
-
-        if (!ret) {
-                g_printerr ("Unable to register display with display manager\n");
-                exit_status = EX_SOFTWARE;
-                goto out;
-        }
-
         ret = spawn_session (state, run_script, state->cancellable);
 
         if (!ret) {
@@ -912,6 +919,8 @@ main (int    argc,
                 goto out;
         }
 
+        g_timeout_add_seconds (2, (GSourceFunc) on_registration_delay_complete, state);
+
         g_main_loop_run (state->main_loop);
 
         /* Only use exit status of session if we're here because it exit */
@@ -921,9 +930,11 @@ main (int    argc,
         }
 
 out:
-        signal_subprocesses (state);
-        wait_on_subprocesses (state);
-        clear_state (&state);
+        if (state != NULL) {
+                signal_subprocesses (state);
+                wait_on_subprocesses (state);
+                clear_state (&state);
+        }
 
         return exit_status;
 }
